import 'package:fis_measure/interfaces/process/calculators/values.dart';
import 'package:fis_measure/process/calcuators/formulas/index.dart' as formulas;

import 'package:vid/us/vid_us_unit.dart';

import '../interfaces/cardiac.dart';
import 'utils.dart';

class CardiacFormulasBase implements ICardiacFormulas {
  @override
  UnitFloatValue edvTech(UnitFloatValue lvidd) {
    var lviddV = lvidd.convert(VidUsUnit.cm);
    var value = formulas.CardiacFormulas.edvTeichholz(lviddV);
    return UnitFloatValue(value, VidUsUnit.cm3);
  }

  @override
  UnitFloatValue esvTech(UnitFloatValue lvids) {
    var lvidsV = lvids.convert(VidUsUnit.cm);
    var value = formulas.CardiacFormulas.esvTeichholz(lvidsV);
    return UnitFloatValue(value, VidUsUnit.cm3);
  }

  @override
  UnitFloatValue edvCube(UnitFloatValue lvidd) {
    var lviddV = lvidd.convert(VidUsUnit.cm);
    var value = formulas.CardiacFormulas.edvCube(lviddV);
    return UnitFloatValue(value, VidUsUnit.cm3);
  }

  @override
  UnitFloatValue esvCube(UnitFloatValue lvids) {
    var lvidsV = lvids.convert(VidUsUnit.cm);
    var value = formulas.CardiacFormulas.esvCube(lvidsV);
    return UnitFloatValue(value, VidUsUnit.cm3);
  }

  @override
  UnitFloatValue ef(UnitFloatValue edv, UnitFloatValue esv) {
    var value = formulas.CardiacFormulas.ef(
      edv.convert(VidUsUnit.cm),
      esv.convert(VidUsUnit.cm),
    );
    return UnitFloatValue(value, VidUsUnit.percent);
  }

  @override
  UnitFloatValue ci(UnitFloatValue sv, {required int hr, required double bsa}) {
    var value = formulas.CardiacFormulas.ci(
      sv.convert(VidUsUnit.cm3),
      hr: hr,
      bsa: bsa,
    );
    return UnitFloatValue(value, VidUsUnit.Lminm2);
  }

  @override
  UnitFloatValue co(UnitFloatValue sv, {required int hr}) {
    var value = formulas.CardiacFormulas.co(
      sv.convert(VidUsUnit.cm3),
      hr: hr,
    );
    return UnitFloatValue(value, VidUsUnit.Lmin);
  }

  @override
  UnitFloatValue fsPercent(UnitFloatValue lvidd, UnitFloatValue lvids) {
    var value = formulas.CardiacFormulas.fsPercent(
      lvidd.convert(VidUsUnit.cm),
      lvids.convert(VidUsUnit.cm),
    );
    return UnitFloatValue(value, VidUsUnit.percent);
  }

  @override
  UnitFloatValue ivsPercent(UnitFloatValue ivss, UnitFloatValue ivsd) {
    var value = formulas.CardiacFormulas.ivsPercent(
      ivss.convert(VidUsUnit.cm),
      ivsd.convert(VidUsUnit.cm),
    );
    return UnitFloatValue(value, VidUsUnit.percent);
  }

  @override
  UnitFloatValue lvdMass(
      UnitFloatValue ivsd, UnitFloatValue lvidd, UnitFloatValue lvpwd) {
    var value = formulas.CardiacFormulas.lvdMass(
      ivsd.convert(VidUsUnit.cm),
      lvidd.convert(VidUsUnit.cm),
      lvpwd.convert(VidUsUnit.cm),
    );
    return UnitFloatValue(value, VidUsUnit.kg);
  }

  @override
  UnitFloatValue lvdMassAL(UnitFloatValue lvadSaxEpi,
      UnitFloatValue lvadSaxEndo, UnitFloatValue lvldApical) {
    var value = formulas.CardiacFormulas.lvdMassAL(
      lvadSaxEpi.convert(VidUsUnit.cm2),
      lvadSaxEndo.convert(VidUsUnit.cm2),
      lvldApical.convert(VidUsUnit.cm),
    );
    return UnitFloatValue(value, VidUsUnit.kg);
  }

  @override
  UnitFloatValue lvdMassIndex(UnitFloatValue lvdmass, {required double bsa}) {
    var value = formulas.CardiacFormulas.lvdMassIndex(
      lvdmass.convert(VidUsUnit.kg),
      bsa,
    );
    return UnitFloatValue(value, VidUsUnit.gm2);
  }

  @override
  UnitFloatValue lvevi(UnitFloatValue lvev, {required double bsa}) {
    var value = formulas.CardiacFormulas.lvevi(
      lvev.convert(VidUsUnit.cm3),
      bsa,
    );
    return UnitFloatValue(value, VidUsUnit.cm3m2);
  }

  @override
  UnitFloatValue lvidIndex(UnitFloatValue lvid, {required double bsa}) {
    var value = formulas.CardiacFormulas.lvevi(
      lvid.convert(VidUsUnit.cm),
      bsa,
    );
    return UnitFloatValue(value, VidUsUnit.None);
  }

  @override
  UnitFloatValue lvidN(UnitFloatValue lvid, UnitFloatValue weight) {
    var value = formulas.CardiacFormulas.lvidN(
      lvid.convert(VidUsUnit.cm),
      weight.convert(VidUsUnit.kg),
    );
    return UnitFloatValue(value, VidUsUnit.None);
  }

  @override
  UnitFloatValue lvpwPercent(UnitFloatValue lvpws, UnitFloatValue lvpwd) {
    var value = formulas.CardiacFormulas.lvpwPercent(
      lvpws.convert(VidUsUnit.cm),
      lvpwd.convert(VidUsUnit.cm),
    );
    return UnitFloatValue(value, VidUsUnit.percent);
  }

  @override
  UnitFloatValue si(UnitFloatValue sv, {required double bsa}) {
    var value = formulas.CardiacFormulas.si(
      sv.convert(VidUsUnit.cm3),
      bsa,
    );
    return UnitFloatValue(value, VidUsUnit.cm3m2);
  }

  @override
  UnitFloatValue sv(UnitFloatValue edv, UnitFloatValue esv) {
    var value = formulas.CardiacFormulas.sv(
      edv.convert(VidUsUnit.cm3),
      esv.convert(VidUsUnit.cm3),
    );
    return UnitFloatValue(value, VidUsUnit.cm3);
  }
}
