import 'package:fis_common/logger/logger.dart';
import 'package:fis_jsonrpc/rpc.dart';
import 'package:fis_measure/interfaces/process/items/item_metas.dart';
import 'package:fis_measure/interfaces/process/items/terms.dart';
import 'package:fis_measure/process/calcuators/urm_calcuators/urm_ellipse_measure.dart';
import 'package:fis_measure/process/workspace/urm/application.dart';
import 'package:vid/us/vid_us_unit.dart';

class URMEllipseDensityCal extends URMEllipseMeasureCal {
  URMEllipseDensityCal(super.ref);

  @override
  void calculate() {}

  @override
  Future<void> calculateAsync() async {
    super.handleURMSimpleMeasureParams();
    try {
      if (ref.feature == null) return;
      if (ref.application is! URMApplication) return;
      final URMApplication urmApplication = ref.application as URMApplication;

      URMMeasureProcessResult? outResult =
          await urmApplication.getURMMeasureResult(
        urmMeasureType: URMMeasureType.URMDen,
        rOIType: URMROIType.URMEllipse,
        srcDPoints: param!.srcDPoints,
      );

      if (outResult != null) {
        final feature = ref.feature!;
        feature.updateStringValue(
            ItemOutputMeta("Placeholder", "Placeholder", VidUsUnit.None),
            "",
            VidUsUnit.None);
        for (var output in ref.meta.outputs) {
          if (output.name == MeasureTerms.URMDenROI) {
            output.unit = VidUsUnit.percent;
            feature.updateFloatValue(
              output,
              outResult.resultData * 100,
              output.unit,
            );
          }
        }
      } else {
        throw Exception("URM Measure API error");
      }
    } catch (e) {
      logger.e('URM Measure error: $e');
      return;
    }
    ref.application.updateRenderReady.emit(this, null);
  }
}
