import 'dart:ui';

import 'package:fis_common/logger/logger.dart';
import 'package:fis_jsonrpc/rpc.dart';
import 'package:fis_measure/interfaces/date_types/point.dart';
import 'package:fis_measure/process/primitives/urm_measure/urm_shell_measure.dart';

// import 'package:fis_measure/process/primitives/urm_straightline.dart';
import 'package:fis_measure/process/workspace/urm/application.dart';

import '../calculator.dart';

class URMShellMeasureCal extends Calculator<URMShellMeasure, double> {
  URMShellMeasureCal(
    URMShellMeasure ref,
  ) : super(ref);
  URMSimpleMeasureParams? param;
  @override
  void calculate() {}

  @override
  Future<void> calculateAsync() async {
    if (ref.feature == null) return;
    try {
      if (ref.application is! URMApplication) {
        return;
      }

      Size urmResultSize = const Size(0, 0);
      final URMApplication urmApplication = ref.application as URMApplication;
      urmResultSize = Size(urmApplication.resultWidth.toDouble(),
          urmApplication.resultHeight.toDouble());
      final List<DPoint> points = ref.feature!.innerPoints
          .map((e) => e.scale2Size(urmResultSize))
          .toList();

      String description = "URM\n Calculating...";
      updateStringValue(description);
    } catch (e) {
      logger.e('URM Measure error: $e');
      return;
    }
  }

  /// 处理获取到的URM参数
  void handleURMSimpleMeasureParams() {
    if (ref.feature == null) return;

    try {
      if (ref.application is! URMApplication) {
        return;
      }
      final URMApplication urmApplication = ref.application as URMApplication;

      // 添加入参
      List<UrmPoint> srcDPoints = [];
      for (var point in ref.feature!.innerPoints) {
        srcDPoints.add(urmApplication.localToView(point));
      }

      param = URMSimpleMeasureParams(
        urmMeasureType: URMMeasureType.URMCurvature,
        rOIType: URMROIType.URMShell,
        srcDPoints: srcDPoints,
      );
    } catch (e) {
      logger.e('URM handleURMSimpleMeasureParams error: $e');
    }
  }
}
