import 'package:fis_common/logger/logger.dart';
import 'package:fis_jsonrpc/services/aIDiagnosis.m.dart';
import 'package:fis_measure/interfaces/process/items/item_metas.dart';
import 'package:fis_measure/interfaces/process/items/terms.dart';
import 'package:fis_measure/process/calcuators/urm_calcuators/urm_trace_measure.dart';
import 'package:fis_measure/process/workspace/urm/application.dart';
import 'package:vid/us/vid_us_unit.dart';

class URMTraceDenVelMeasureCal extends URMTraceMeasureCal {
  URMTraceDenVelMeasureCal(super.ref);

  @override
  void calculate() {}

  @override
  Future<void> calculateAsync() async {
    super.handleURMSimpleMeasureParams();
    try {
      if (ref.feature == null) return;
      if (ref.application is! URMApplication) return;
      final URMApplication urmApplication = ref.application as URMApplication;

      URMMeasureProcessResult? outresult =
          await urmApplication.getURMMeasureResult(
        urmMeasureType: URMMeasureType.URMDenVelMeasure,
        rOIType: URMROIType.URMTrace,
        srcDPoints: param!.srcDPoints,
      );
      if (outresult == null) {
        return;
      }
      URMDenMeasureResult? denresult = outresult.denMeasureResult;
      URMVelMeasureResult? velresult = outresult.velMeasureResult;
      if (denresult != null && velresult != null) {
        final feature = ref.feature!;
        feature.updateStringValue(
            ItemOutputMeta("Placeholder", "Placeholder", VidUsUnit.None),
            "",
            VidUsUnit.None);
        for (var output in ref.meta.outputs) {
          if (output.name == MeasureTerms.URMDenROI) {
            output.unit = VidUsUnit.percent;
            feature.updateFloatValue(
                output, denresult.roiDen * 100, output.unit);
          } else if (output.name == MeasureTerms.URMDenFractalDim) {
            output.unit = VidUsUnit.None;
            feature.updateFloatValue(
                output, denresult.roiFractalDim, output.unit);
          } else if (output.name == MeasureTerms.URMDenMax) {
            output.unit = VidUsUnit.None;
            feature.updateFloatValue(output, denresult.maxDensity, output.unit);
          } else if (output.name == MeasureTerms.URMDenMin) {
            output.unit = VidUsUnit.None;
            feature.updateFloatValue(output, denresult.minDensity, output.unit);
          } else if (output.name == MeasureTerms.URMDenMean) {
            output.unit = VidUsUnit.None;
            feature.updateFloatValue(
                output, denresult.meanDensity, output.unit);
          } else if (output.name == MeasureTerms.URMDenStd) {
            output.unit = VidUsUnit.None;
            feature.updateFloatValue(
                output, denresult.varianceDensity, output.unit);
          } else if (output.name == MeasureTerms.URMVelMax) {
            output.unit = VidUsUnit.mms;
            feature.updateFloatValue(output, velresult.maxVel, output.unit);
          } else if (output.name == MeasureTerms.URMVelMin) {
            output.unit = VidUsUnit.mms;
            feature.updateFloatValue(output, velresult.minVel, output.unit);
          } else if (output.name == MeasureTerms.URMVelMean) {
            output.unit = VidUsUnit.mms;
            feature.updateFloatValue(output, velresult.meanVel, output.unit);
          } else if (output.name == MeasureTerms.URMVelStd) {
            output.unit = VidUsUnit.mms;
            feature.updateFloatValue(
                output, velresult.varianceVel, output.unit);
          } else if (output.name == MeasureTerms.Area) {
            output.unit = VidUsUnit.cm2;
            feature.updateFloatValue(output, velresult.roiArea, output.unit);
          }
        }
      } else {
        throw Exception("URM Measure API error");
      }
    } catch (e) {
      logger.e('URM Measure error: $e');
      return;
    }
    ref.application.updateRenderReady.emit(this, null);
  }
}
