import 'dart:ui';

import 'package:fis_measure/interfaces/process/items/item.dart';
import 'package:fis_measure/interfaces/process/items/item_metas.dart';
import 'package:fis_measure/process/calcuators/urm_calcuators/urm_den.dart';
import 'package:fis_measure/process/items/top_item_feature.dart';
import 'package:fis_measure/process/primitives/straightline.dart';
import 'package:fis_measure/process/primitives/urm_measure/urm_rect_measure.dart';
import 'package:fis_measure/process/primitives/urm_measure/urm_straight_curvature_line_measure.dart';
import 'package:fis_measure/process/items/item.dart';
import 'package:fis_measure/process/items/item_feature.dart';
import 'package:fis_measure/process/items/top_item.dart';
// import 'package:fis_measure/process/primitives/urm_straightline.dart';

class URMDen extends URMDenAbstract<URMDenFeature> {
  static const String _area1Key = "URMCurvature";
  static const String _area2Key = "URMRectDensity";
  static const String _area3Key = "URMRectFractalDim";
  static const String _area4Key = "Vessel diameter";
  static const String _area5Key = "Vessel distance";

  late final URMStraightCurvatureLineMeasure a1;
  late final URMRectMeasure a2;
  late final URMRectMeasure a3;
  late final StraightLine a4;
  late final StraightLine a5;

  URMDen(ItemMeta meta) : super(meta) {
    final metaA1 = meta.getChildByName(_area1Key)!;
    final metaA2 = meta.getChildByName(_area2Key)!;
    final metaA3 = meta.getChildByName(_area3Key)!;
    final metaA4 = meta.getChildByName(_area4Key)!;
    final metaA5 = meta.getChildByName(_area5Key)!;
    a1 = URMStraightCurvatureLineMeasure.createURMStraightCurvatureLineMeasure(
        metaA1, this);
    a2 = URMRectMeasure.createURMRectDensity(metaA2, this);
    a3 = URMRectMeasure.createURMRectFractalDim(metaA3, this);
    a4 = StraightLine.createDistance(metaA4, this);
    a5 = StraightLine.createDistance(metaA5, this);
    childItems.add(a1);
    childItems.add(a2);
    childItems.add(a3);
    childItems.add(a4);
    childItems.add(a5);
  }

  @override
  StraightLine get child1 => a1;

  @override
  URMRectMeasure get child2 => a2;

  @override
  URMRectMeasure get child3 => a3;

  @override
  StraightLine get child4 => a4;

  @override
  StraightLine get child5 => a5;

  @override
  URMDenFeature buildFeature() => URMDenFeature(this);

  @override
  void onCancelingOnce() {}

  static URMDen create(ItemMeta meta, [IMeasureItem? parent]) {
    var urmDen = URMDen(meta);
    urmDen.calculator = URMDenCal(urmDen);

    return urmDen;
  }
}

abstract class URMDenAbstract<T extends MeasureItemFeature>
    extends TopMeasureItem<T> {
  URMDenAbstract(ItemMeta meta) : super(meta);

  MeasureItem get child1;
  MeasureItem get child2;
  MeasureItem get child3;
  MeasureItem get child4;
  MeasureItem get child5;

  @override
  bool get finishAfterUnactive => true;
}

class URMDenFeature extends TopMeasureItemFeature {
  URMDenFeature(
    ITopMeasureItem refItem,
  ) : super(refItem);

  @override
  void setZoomRect(Rect? rect) {
    zoomRect = rect;
    for (var item in refItem.childItems) {
      item.feature?.setZoomRect(rect);
    }
  }
}
