import 'dart:math';
import 'dart:ui';

import 'package:fis_measure/view/ai_result_modifier/index.dart';
import 'package:flutter/material.dart';
import 'package:path_drawing/path_drawing.dart';

class AIResultCanvas extends CustomPainter {
  final List<Offset> contoursPoints;
  final List<Offset> newContoursPoints;
  final List<Offset> keyPoints;
  final List<Offset> highlightKeyPoints;
  final List<int> lesionSizePointsIndexes;
  final List<int> penModeKeyPointIndexes;

  final AiResultModifierMode currMode;

  AIResultCanvas({
    required this.contoursPoints,
    required this.newContoursPoints,
    required this.keyPoints,
    required this.highlightKeyPoints,
    required this.currMode,
    required this.lesionSizePointsIndexes,
    required this.penModeKeyPointIndexes,
  });

  /// 绘制基础轮廓
  void handleBase(Canvas canvas) {
    // 设置画笔
    final contoursPaint = Paint()
      ..color = const Color.fromARGB(255, 218, 165, 32)
      ..strokeCap = StrokeCap.round
      ..strokeWidth = 2.0
      ..style = PaintingStyle.stroke;

    // 为每个点创建路径，并绘制轮廓
    canvas.drawPoints(PointMode.polygon, contoursPoints, contoursPaint);
    // 连接起点终点
    if (contoursPoints.isNotEmpty) {
      Path path = Path();
      path.moveTo(contoursPoints[0].dx, contoursPoints[0].dy);
      path.lineTo(contoursPoints[contoursPoints.length - 1].dx,
          contoursPoints[contoursPoints.length - 1].dy);
      canvas.drawPath(path, contoursPaint);
    }

    // 连接横纵比线段
    if (lesionSizePointsIndexes.length >= 2) {
      final lesionSizeLinePaint = Paint()
        ..color = const Color.fromARGB(255, 187, 250, 255)
        ..strokeCap = StrokeCap.round
        ..strokeWidth = 2.0
        ..style = PaintingStyle.stroke;

      Path path = Path();
      path.moveTo(keyPoints[lesionSizePointsIndexes[0]].dx,
          keyPoints[lesionSizePointsIndexes[0]].dy);
      path.lineTo(keyPoints[lesionSizePointsIndexes[1]].dx,
          keyPoints[lesionSizePointsIndexes[1]].dy);
      canvas.drawPath(path, lesionSizeLinePaint);
    }
    if (lesionSizePointsIndexes.length >= 4) {
      final lesionSizeLinePaint = Paint()
        ..color = const Color.fromARGB(255, 255, 254, 187)
        ..strokeCap = StrokeCap.round
        ..strokeWidth = 2.0
        ..style = PaintingStyle.stroke;

      Path path = Path();
      path.moveTo(keyPoints[lesionSizePointsIndexes[2]].dx,
          keyPoints[lesionSizePointsIndexes[2]].dy);
      path.lineTo(keyPoints[lesionSizePointsIndexes[3]].dx,
          keyPoints[lesionSizePointsIndexes[3]].dy);
      canvas.drawPath(path, lesionSizeLinePaint);
    }
  }

  /// 拖拽模式
  void handleDragMode(Canvas canvas) {
    // 绘制关键点，每个关键点处打个绿点
    final keyPointPaint = Paint()
      ..color = const Color.fromARGB(255, 143, 188, 143)
      ..strokeCap = StrokeCap.round
      ..strokeWidth = 3.0;
    if (keyPoints.isNotEmpty) {
      for (int i = 0; i < keyPoints.length; i++) {
        if (lesionSizePointsIndexes.contains(i)) {
          paintX(
            canvas,
            keyPoints[i],
            6.0,
            3.0,
            const Color.fromARGB(255, 235, 166, 62),
          );
        } else {
          canvas.drawCircle(
              Offset(keyPoints[i].dx, keyPoints[i].dy), 3.0, keyPointPaint);
        }
      }
    }

    // 绘制高亮关键点，每个关键点处打个红点
    final highlightKeyPointsPaint = Paint()
      ..color = const Color.fromARGB(255, 255, 81, 81)
      ..strokeCap = StrokeCap.round
      ..strokeWidth = 3.0;
    if (highlightKeyPoints.isNotEmpty) {
      for (int i = 0; i < highlightKeyPoints.length; i++) {
        canvas.drawCircle(
            Offset(highlightKeyPoints[i].dx, highlightKeyPoints[i].dy),
            3.5,
            highlightKeyPointsPaint);
      }

      /// 当选中横纵顶点时，叠加特殊处理
      if (highlightKeyPoints.length == 1) {
        // 遍历 lesionSizePointsIndexes 查找是否有跟关键点重合的顶点
        for (int i = 0; i < lesionSizePointsIndexes.length; i++) {
          final xPoint = keyPoints[lesionSizePointsIndexes[i]];
          if (xPoint == highlightKeyPoints[0]) {
            paintX(
              canvas,
              xPoint,
              6.0,
              3.5,
              const Color.fromARGB(255, 255, 81, 81),
            );
          }
        }
      }
    }
  }

  /// 画轮廓模式
  void handlePenMode(Canvas canvas) {
    // 遍历 penModeKeyPointIndexes 绘制高亮点
    final highlightKeyPointsPaint = Paint()
      ..color = const Color.fromARGB(255, 96, 255, 81)
      ..strokeWidth = 10.0;
    for (int i = 0; i < penModeKeyPointIndexes.length; i++) {
      int index = penModeKeyPointIndexes[i];
      if (index >= 0 && index < contoursPoints.length) {
        canvas.save();
        canvas.translate(contoursPoints[index].dx, contoursPoints[index].dy);
        canvas.rotate(pi / 4);
        canvas.drawRect(
            Rect.fromCenter(
                center: const Offset(0.0, 0.0), width: 10.0, height: 10.0),
            highlightKeyPointsPaint);
        canvas.restore();
      }
    }

    // 设置画笔
    final newContoursPaint = Paint()
      ..color = const Color.fromARGB(255, 32, 218, 103)
      ..strokeCap = StrokeCap.round
      ..strokeWidth = 2.0
      ..style = PaintingStyle.stroke;
    canvas.drawPoints(PointMode.polygon, newContoursPoints, newContoursPaint);
  }

  /// 截图模式
  void handleScreenshotMode(Canvas canvas) {
    // 设置虚线圆点画笔
    final contoursPaint = Paint()
      ..color = Colors.green
      ..strokeCap = StrokeCap.round
      ..strokeWidth = 3.0
      ..style = PaintingStyle.stroke;

    // 遍历 contoursPoints 绘制轮廓
    if (contoursPoints.isNotEmpty) {
      Path path = Path();
      path.moveTo(contoursPoints[0].dx, contoursPoints[0].dy);
      for (int i = 1; i < contoursPoints.length; i++) {
        path.lineTo(contoursPoints[i].dx, contoursPoints[i].dy);
      }
      path.close();
      canvas.drawPath(
          dashPath(
            path,
            dashArray: CircularIntervalList<double>([1, 10]),
          ),
          contoursPaint);
    }
    // 连接横纵比线段
    if (lesionSizePointsIndexes.length >= 2) {
      Path path = Path();
      path.moveTo(keyPoints[lesionSizePointsIndexes[0]].dx,
          keyPoints[lesionSizePointsIndexes[0]].dy);
      path.lineTo(keyPoints[lesionSizePointsIndexes[1]].dx,
          keyPoints[lesionSizePointsIndexes[1]].dy);
      canvas.drawPath(
          dashPath(
            path,
            dashArray: CircularIntervalList<double>([1, 5]),
          ),
          contoursPaint);
    }
    if (lesionSizePointsIndexes.length >= 4) {
      Path path = Path();
      path.moveTo(keyPoints[lesionSizePointsIndexes[2]].dx,
          keyPoints[lesionSizePointsIndexes[2]].dy);
      path.lineTo(keyPoints[lesionSizePointsIndexes[3]].dx,
          keyPoints[lesionSizePointsIndexes[3]].dy);
      canvas.drawPath(
          dashPath(
            path,
            dashArray: CircularIntervalList<double>([1, 5]),
          ),
          contoursPaint);
    }
    if (keyPoints.isNotEmpty) {
      for (int i = 0; i < keyPoints.length; i++) {
        if (lesionSizePointsIndexes.contains(i)) {
          paintX(
            canvas,
            keyPoints[i],
            6.0,
            3,
            Colors.green,
          );
        }
      }
    }
  }

  @override
  void paint(Canvas canvas, Size size) {
    // print("绘制一次 $currMode");
    switch (currMode) {
      case AiResultModifierMode.pen:
        handleBase(canvas);
        handlePenMode(canvas);
        break;
      case AiResultModifierMode.drag:
        handleBase(canvas);
        handleDragMode(canvas);
        break;
      case AiResultModifierMode.screenshot:
        handleScreenshotMode(canvas);
        break;
      default:
    }
  }

  /// 绘制叉叉
  void paintX(
      Canvas canvas, Offset center, double radius, double width, Color color) {
    final paint = Paint()
      ..color = color
      ..strokeCap = StrokeCap.round
      ..strokeWidth = width
      ..style = PaintingStyle.stroke;

    Path path = Path();
    path.moveTo(center.dx - radius, center.dy - radius);
    path.lineTo(center.dx + radius, center.dy + radius);
    path.moveTo(center.dx + radius, center.dy - radius);
    path.lineTo(center.dx - radius, center.dy + radius);
    canvas.drawPath(path, paint);
  }

  @override
  bool shouldRepaint(covariant CustomPainter oldDelegate) => true;
}
