import 'package:fis_ui/index.dart';
import 'package:fis_ui/interface/interactive_container.dart';
import 'package:flutter/material.dart';

class FCustomOperateButton extends FStatefulWidget
    implements FInteractiveContainer {
  final String text;
  final VoidCallback onPressed;
  final IconData? icon;
  final Color? normalColor;
  final Color? highlightColor;

  const FCustomOperateButton({
    required this.text,
    required this.onPressed,
    this.icon,
    this.normalColor,
    this.highlightColor,
    super.key,
  });

  @override
  FState<FStatefulWidget> createState() {
    return _CustomButtonState();
  }

  @override
  String get pageName => "custom operate button";
}

class _CustomButtonState extends FState<FCustomOperateButton> {
  bool hover = false;
  @override
  FWidget build(BuildContext context) {
    Color highlightColor = widget.highlightColor ?? Colors.redAccent;
    return FMouseRegion(
      onEnter: (event) {
        setState(() {
          hover = true;
        });
      },
      onExit: (event) {
        setState(() {
          hover = false;
        });
      },
      child: FElevatedButton(
        onPressed: widget.onPressed,
        name: 'FCustomButton',
        businessParent: widget,
        style: ElevatedButton.styleFrom(
          padding: const EdgeInsets.symmetric(horizontal: 8, vertical: 16),
          backgroundColor:
              hover ? highlightColor : widget.normalColor ?? Colors.transparent,
          side: BorderSide(color: highlightColor, width: 1),
          shape: RoundedRectangleBorder(
            borderRadius: BorderRadius.circular(8),
          ),
        ),
        child: FRow(
          mainAxisAlignment: MainAxisAlignment.center,
          children: [
            FText(
              widget.text,
              style: TextStyle(
                color: hover ? Colors.grey[300] : highlightColor,
              ),
            ),
            const FSizedBox(width: 3),
            FIcon(
              widget.icon ?? Icons.exit_to_app,
              color: hover ? Colors.grey[300] : highlightColor,
            ),
          ],
        ),
      ),
    );
  }
}
