import 'package:fis_i18n/i18n.dart';
import 'package:fis_jsonrpc/rpc.dart';
import 'package:fis_measure/process/language/measure_language.dart';
import 'package:fis_measure/process/workspace/measure_data_controller.dart';
import 'package:fis_theme/theme.dart';
import 'package:fis_ui/index.dart';
import 'package:fis_ui/interface/interactive_container.dart';
import 'package:flutter/material.dart';

import 'package:get/get.dart';

///选中设置对话框
class HasSelectedSettingDialog extends FStatelessWidget
    implements FInteractiveContainer {
  const HasSelectedSettingDialog({
    Key? key,
    required this.itemMeta,
  }) : super(key: key);
  final ItemMetaDTO itemMeta;

  @override
  final String pageName = 'HasSelectedSettingDialog';
  @override
  FWidget build(BuildContext context) {
    return FSimpleDialog(
      title: FText(
        i18nBook.measure.selectedMeasureToolConfig.t,
        style: const TextStyle(
          color: Colors.white,
          fontSize: 18,
        ),
      ),
      isDefault: false,
      bottomButton: FContainer(
        padding: const EdgeInsets.symmetric(vertical: 10),
        child: FRow(
          mainAxisAlignment: MainAxisAlignment.center,
          children: [
            _buildCommonButton(i18nBook.common.back.t, () {
              Get.back();
            }),
          ],
        ),
      ),
      children: [
        SizedBox(
          width: 830,
          height: 500,
          child: _DialogPage(parent: this, itemMeta: itemMeta),
        ),
      ],
    );
  }

  FWidget _buildCommonButton(String buttonName, VoidCallback onPressed) {
    return FElevatedButton(
      child: FText(buttonName),
      onPressed: () {
        onPressed.call();
      },
      businessParent: this,
      name: buttonName,
      style: ElevatedButton.styleFrom(
        elevation: 0,
        padding: const EdgeInsets.symmetric(
          vertical: 15,
          horizontal: 25,
        ),
      ),
    );
  }
}

///对话框界面
class _DialogPage extends StatefulWidget {
  ///父级节点
  final FInteractiveContainer parent;
  const _DialogPage({
    required this.parent,
    required this.itemMeta,
  });

  final ItemMetaDTO itemMeta;

  @override
  State<_DialogPage> createState() => _DialogPageState();
}

class _DialogPageState extends State<_DialogPage> {
  /// 测量语言包
  final measureLanguage = MeasureLanguage();
  final measureData = Get.find<MeasureDataController>();

  /// 左侧tab列表
  late List<ChildItemMetaDTO> selectChildItemMeta =
      widget.itemMeta.multiMethodItems ?? [];

  /// 测量方式
  late List<ChildItemMetaDTO> multiMethodItems =
      widget.itemMeta.multiMethodItems ?? [];

  /// 判断是否有多种测量方式
  late List<OutputItemMetaDTO> availableOutputs =
      (widget.itemMeta.multiMethodItems != null &&
              widget.itemMeta.multiMethodItems!.isNotEmpty)
          ? widget.itemMeta.multiMethodItems?.first.calculator
                  ?.availableOutputs ??
              []
          : (widget.itemMeta.calculator?.availableOutputs ?? []);

  late int selectedTabIndex = -1;

  @override
  void initState() {
    super.initState();
    setTabList();
  }

  @override
  void dispose() {
    if (!measureData.itemMetaListConfig.contains(widget.itemMeta)) {
      measureData.itemMetaListConfig.add(widget.itemMeta);
    }
    super.dispose();
  }

  void setTabList() {
    List<ChildItemMetaDTO> tabList = [];
    List<ChildItemMetaDTO> childTabList = [];

    /// 最下面的子集
    List<ChildItemMetaDTO> selectChildItem = [];
    tabList = (widget.itemMeta.multiMethodItems ?? [])
            .firstWhereOrNull((element) => element.isWorking)
            ?.childItems ??
        [];

    for (var element in tabList) {
      childTabList.add(element);
      selectChildItem = element.childItems ?? [];
      childTabList.addAll(selectChildItem
              .firstWhereOrNull((element) =>
                  element.isWorking &&
                  element.childItems != null &&
                  element.childItems!.isNotEmpty)
              ?.childItems ??
          []);
    }
    selectChildItemMeta = childTabList;
    setState(() {});
  }

  // final controller = Get.put(MeasureConfigurationController());
  @override
  FWidget build(BuildContext context) {
    return FRow(
      children: [
        _buildLeftMeasureMethod(),
        FExpanded(child: _buildRightMeasurementResultAndUnit()),
      ],
    );
  }

  /// 左侧测量方法
  FWidget _buildLeftMeasureMethod() {
    return FContainer(
      width: 150,
      decoration: BoxDecoration(
        border: Border(
          left: BorderSide(
            color: FTheme.ins.colorScheme.line,
          ),
        ),
        color: Colors.grey[300],
      ),
      child: FColumn(
        children: [
          FContainer(
            width: double.infinity,
            color: selectedTabIndex == -1
                ? FTheme.ins.colorScheme.primary
                : Colors.white,
            height: 60,
            child: FInkWell(
              child: FCenter(
                child: FText(
                  widget.itemMeta.description ?? '',
                  style: TextStyle(
                    color: selectedTabIndex == -1 ? Colors.white : Colors.black,
                  ),
                ),
              ),
              onTap: () {
                selectedTabIndex = -1;
                // availableOutputs =
                multiMethodItems = widget.itemMeta.multiMethodItems ?? [];
                if (widget.itemMeta.multiMethodItems?.isNotEmpty ?? false) {
                  availableOutputs = widget.itemMeta.multiMethodItems?.first
                          .calculator?.availableOutputs ??
                      [];
                } else {
                  availableOutputs =
                      widget.itemMeta.calculator?.availableOutputs ?? [];
                }

                setState(() {});
              },
            ),
          ),
          FColumn(
            children: selectChildItemMeta.map((e) {
              int index = selectChildItemMeta.indexOf(e);
              return FContainer(
                width: double.infinity,
                height: 60,
                color: selectedTabIndex == index
                    ? FTheme.ins.colorScheme.primary
                    : Colors.white,
                child: FInkWell(
                  child: FCenter(
                      child: FText(e.description ?? e.name ?? '',
                          style: TextStyle(
                            color: selectedTabIndex == index
                                ? Colors.white
                                : Colors.black,
                          ))),
                  onTap: () {
                    selectedTabIndex = index;
                    multiMethodItems = e.childItems ?? [];
                    if (multiMethodItems.isNotEmpty) {
                      availableOutputs =
                          multiMethodItems.first.calculator?.availableOutputs ??
                              [];
                    } else {
                      availableOutputs = e.calculator?.availableOutputs ?? [];
                    }

                    setState(() {});
                  },
                ),
              );
            }).toList(),
          ),
        ],
      ),
    );
  }

  /// 计算结果和单位
  FWidget _buildRightMeasurementResultAndUnit() {
    return FListView(
      shrinkWrap: true,
      children: [
        FColumn(
          mainAxisSize: MainAxisSize.max,
          mainAxisAlignment: MainAxisAlignment.start,
          children: [
            if (multiMethodItems.isNotEmpty) _buildMeasureResult(),
            if (availableOutputs.isNotEmpty) _buildMeasureUnit(),
          ],
        ),
      ],
    );
  }

  FWidget _buildTitleContainer(String title, FWidget wrapList) {
    return FColumn(
      crossAxisAlignment: CrossAxisAlignment.start,
      children: [
        FContainer(
          margin: const EdgeInsets.all(15),
          child: FText(title),
        ),
        FContainer(
          padding: const EdgeInsets.only(
            left: 35,
            right: 25,
            top: 25,
            bottom: 25,
          ),
          margin: const EdgeInsets.symmetric(
            horizontal: 15,
          ),
          width: double.infinity,
          decoration: BoxDecoration(
            border: Border.all(color: FTheme.ins.colorScheme.line),
            borderRadius: BorderRadius.circular(8),
          ),
          child: wrapList,
        ),
      ],
    );
  }

  FWidget _buildMeasureResult() {
    return _buildTitleContainer(
      i18nBook.measure.selectMeasureMethod.t,
      _buildResultWrapList(multiMethodItems),
    );
  }

  FWidget _buildMeasureUnit() {
    return _buildTitleContainer(
      i18nBook.measure.selectCalculateResultAndUnit.t,
      _buildUnitWrapList(availableOutputs),
    );
  }

  /// 这边是测量结果
  FWidget _buildResultWrapList(List<ChildItemMetaDTO> list) {
    return FWrap(
      spacing: 20.0, // 主轴(水平)方向间距
      runSpacing: 10.0, // 纵轴（垂直）方向间距
      direction: Axis.horizontal,
      crossAlignment: WrapCrossAlignment.center,
      alignment: WrapAlignment.start,
      children: list
          .map(
            (e) => _buildResultItem(e.description ?? e.name ?? '', e),
          )
          .toList(),
    );
  }

  /// 这边是测量单位
  FWidget _buildUnitWrapList(List<OutputItemMetaDTO> list) {
    return FWrap(
      spacing: 20.0, // 主轴(水平)方向间距
      runSpacing: 10.0, // 纵轴（垂直）方向间距
      direction: Axis.horizontal,
      crossAlignment: WrapCrossAlignment.center,
      alignment: WrapAlignment.start,
      children: list.map((e) {
        int _index = list.indexOf(e);
        bool _isCanClick = true;
        if (_index == 0) {
          _isCanClick = false;
        }
        return _buildUnitItem(e.description ?? e.name ?? '', e, _isCanClick);
      }).toList(),
    );
  }

  FWidget _buildResultItem(String name, ChildItemMetaDTO childItemMeta) {
    return FGestureDetector(
      name: name,
      businessParent: widget.parent,
      onTap: () {
        if (!childItemMeta.isWorking) {
          for (var element in multiMethodItems) {
            element.isWorking = false;
          }
          childItemMeta.isWorking = true;
          setTabList();
        }
        setState(() {});
      },
      child: FContainer(
        width: 180,
        height: 65,
        decoration: BoxDecoration(
          color: childItemMeta.isWorking
              ? FTheme.ins.colorScheme.primary
              : Colors.white,
          border: Border.all(
            color: FTheme.ins.colorScheme.line,
          ),
        ),
        child: FCenter(
          child: FColumn(
            mainAxisAlignment: MainAxisAlignment.center,
            children: [
              FText(
                name,
                style: TextStyle(
                  color: childItemMeta.isWorking ? Colors.white : Colors.black,
                ),
              ),
            ],
          ),
        ),
      ),
    );
  }

  FWidget _buildUnitItem(
      String name, OutputItemMetaDTO outputItemMeta, bool isCanClick) {
    return FGestureDetector(
      name: name,
      businessParent: widget.parent,
      onTap: () {
        if (isCanClick) {
          outputItemMeta.isWorking ??= false;
          outputItemMeta.isWorking = !outputItemMeta.isWorking!;
          setState(() {});
        }
      },
      child: FContainer(
        width: 180,
        height: 65,
        decoration: BoxDecoration(
          color: outputItemMeta.isWorking ?? false
              ? FTheme.ins.colorScheme.primary
              : Colors.white,
          border: Border.all(
            color: FTheme.ins.colorScheme.line,
          ),
        ),
        child: FCenter(
          child: FColumn(
            mainAxisAlignment: MainAxisAlignment.center,
            children: [
              FText(
                name,
                style: TextStyle(
                  color: outputItemMeta.isWorking ?? false
                      ? Colors.white
                      : Colors.black,
                ),
              ),
            ],
          ),
        ),
      ),
    );
  }
}
