// ignore: must_be_immutable

import 'package:fis_common/extensions/index.dart';
import 'package:fis_common/logger/logger.dart';
import 'package:fis_i18n/i18n.dart';
import 'package:fis_jsonrpc/rpc.dart';
import 'package:fis_measure/process/language/measure_language.dart';
import 'package:fis_measure/process/workspace/measure_data_controller.dart';
import 'package:fis_measure/process/workspace/rpc_bridge.dart';
import 'package:fis_measure/utils/prompt_box.dart';
import 'package:fis_measure/view/loadding/loadding.dart';
import 'package:fis_measure/view/measure/measure_config/measure_configuation_page.dart';
import 'package:fis_measure/view/measure/measure_config/widgets/code_field.dart';
import 'package:fis_measure/view/measure/measure_config/widgets/form_validator.dart';
import 'package:fis_measure/view/measure/measure_config/widgets/state.dart';
import 'package:fis_measure/view/measure/measure_config/widgets/text_field.dart';
import 'package:fis_ui/index.dart';
import 'package:fis_theme/theme.dart';
import 'package:fis_ui/interface/interactive_container.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:get/get.dart';

class SelectModulePage extends FStatefulWidget
    implements FInteractiveContainer {
  final List<ItemMetaGroup> itemMetaListGroup;

  /// 选中的测量项
  List<String> chooseMeasureList = [];

  @override
  final String pageName = 'SelectModulePage';
  SelectModulePage({
    super.key,
    required this.itemMetaListGroup,
    required this.chooseMeasureList,
  });
  @override
  FState<SelectModulePage> createState() => _SelectModulePageState();
}

class _SelectModulePageState extends FState<SelectModulePage> {
  final spaceBetween = const FSizedBox(height: 14);

  /// 数据
  final measureData = Get.find<MeasureDataController>();

  /// 测量语言包
  final measureLanguage = MeasureLanguage();

  static const Color childButtonHighlight = Color.fromRGBO(84, 144, 249, 1);

  /// 数据状态
  final state = SignUpState();

  /// 表单编辑状态
  final formEditState = SignUpFormEditState();

  /// 表单验证状态
  late final formValidateState = SignUpFormValidateState(state);

  // 文本框控制器
  final accountTextController = TextEditingController();
  final verificationCodeTextController = TextEditingController();

  // 文本框焦点
  final accountFocusNode = FocusNode();
  final verificationCodeFocusNode = FocusNode();

  //  表单验证控制器
  final accountValidatorController = FFormItemValidatorController();

  /// 新增测量项目
  ///
  /// [chooseMeasure] 选中的测量项
  void addChooseMeasure(String chooseMeasure) {
    widget.chooseMeasureList.add(
      chooseMeasure,
    );
    setState(() {});
  }

  /// 移除测量项目
  ///
  /// [chooseMeasure] 移除的测量项
  void removeChooseMeasure(String chooseMeasure) {
    widget.chooseMeasureList.remove(
      chooseMeasure,
    );
    setState(() {});
  }

  /// 展示账号验证器
  void showAccountValidator() {
    if (formValidateState.accountBeOccupied) {
      accountValidatorController.hide();
      return;
    }
    if (state.account.isEmpty) {
      accountValidatorController.resetValidate();
    } else {
      accountValidatorController.validate();
    }
  }

  @override
  void initState() {
    super.initState();
  }

  @override
  FWidget build(BuildContext context) {
    return FColumn(
      children: widget.itemMetaListGroup
          .map(
            (e) => FColumn(
              crossAxisAlignment: CrossAxisAlignment.start,
              children: _buildMeasureConfiguration(e),
            ),
          )
          .toList(),
    );
  }

  List<FWidget> _buildMeasureConfiguration(ItemMetaGroup e) {
    return [
      FInkWell(
        onTap: () {
          e.isExpend = !e.isExpend!;
          setState(() {});
        },
        child: FContainer(
          padding: const EdgeInsets.all(15),
          child: _buildGroupTitleRow(
            e.itemMetaGroupTitle.toString(),
            e.isExpend!,
          ),
        ),
      ),
      e.isExpend! ? _buildGroupBody(e) : FContainer(),
    ];
  }

  FWidget _buildGroupTitleRow(
    String itemMetaGroupTitle,
    bool isExpend,
  ) {
    return FRow(
      mainAxisAlignment: MainAxisAlignment.spaceBetween,
      children: [
        FRow(
          children: [
            FText(
              itemMetaGroupTitle,
            ),
            FText(
              measureLanguage.t('measure', itemMetaGroupTitle),
            ),
          ],
        ),
        FRow(
          children: [
            /// TODO baka 等接口好了改
            if (itemMetaGroupTitle == "VET" && false)
              QuickFWidget(
                ElevatedButton(
                  onPressed: () {
                    Get.dialog(FSimpleDialog(
                      title: const FText(
                        /// TODO baka 需要新增翻译
                        "申请试用",
                        style: TextStyle(
                          color: Colors.white,
                          fontSize: 18,
                        ),
                      ),
                      cancelString: i18nBook.common.cancel.t,
                      okString: i18nBook.common.confirm.t,
                      isDefault: true,
                      onOk: () {
                        /// TODO baka 需要新增接口
                      },
                      onCancel: () {
                        Get.back();
                      },
                      onClose: () {
                        Get.back();
                      },
                      children: [
                        FContainer(
                          width: 600,
                          height: 500,
                          padding: const EdgeInsets.symmetric(
                            horizontal: 50,
                            vertical: 40,
                          ),
                          child: _buildPhoneAndVerify(),
                        ),
                      ],
                    ));
                  },

                  /// TODO baka 新加翻译
                  child: const Text("申请试用"),
                ),
              ),
            isExpend
                ? const FIcon(
                    Icons.keyboard_arrow_down_sharp,
                  )
                : const FIcon(
                    Icons.keyboard_arrow_right,
                  ),
          ],
        ),
      ],
    );
  }

  void _onBuildAccountFieldClick(String val) {
    state.account = val;
    formValidateState.accountBeOccupied = false;
    showAccountValidator();
    accountValidatorController.removeCustomError();
  }

  FWidget _buildPhoneAndVerify() {
    return FColumn(
      children: [
        _buildAccountField(),
        spaceBetween,
        QuickFWidget(
          _buildVerificationCodeField(),
        ),
      ],
    );
  }

  FWidget _buildAccountField() {
    final validateState = formValidateState;
    final txtField = Obx(
      () => FBorderTextFormField(
        controller: accountTextController,
        focusNode: accountFocusNode,
        maxLength: 254,
        inError: !validateState.accountChecked && formEditState.accountEdited,
        onChanged: (val) => _onBuildAccountFieldClick(
          val,
        ),
        inputFormatters: [
          FilteringTextInputFormatter.deny(RegExp("[\u4e00-\u9fa5]")),
          FilteringTextInputFormatter.deny(RegExp("\\s")),
        ],

        /// TODO baka 新加翻译
        hintText: "请输入手机号",
      ),
    );

    return FFormItemValidator(
      key: const ValueKey("Signup.FormItem.Account"),
      child: QuickFWidget(txtField),
      // overlayContext: _parentContext,
      controller: accountValidatorController,
      validateSteps: [
        FFormItemValidateStep(
          validateFn: () => validateState.accountFormatCheck,

          /// TODO baka 新加翻译
          tips: "请输入正确的手机号",
        ),
      ],
    );
  }

  Widget _buildVerificationCodeField() {
    if (!Get.isRegistered<VerificationCodeFieldController>(
        tag: "request_trial")) {
      Get.lazyPut(
        () => VerificationCodeFieldController(countDown: 120),
        tag: "request_trial",
      );
    }
    return Obx(
      () => VerificationCodeField(
        key: const ValueKey("Signup.FormItem.VerificationCode"),
        tag: "request_trial",
        textFocusNode: verificationCodeFocusNode,
        textController: verificationCodeTextController,
        onTextChanged: (val) => state.verificationCode = val,
        inError: !formValidateState.verificationCodeChecked &&
            formEditState.verificationCodeEdited,
        fetchCodeFunc: onFetchCodeClick,
      ),
    );
  }

  /// 表单失焦
  void unfocusForm() {
    accountFocusNode.unfocus();
    verificationCodeFocusNode.unfocus();
  }

  /// 隐藏表单验证提示
  void hideFormValidator() {
    accountValidatorController.hide();
  }

  /// 点击获取验证码
  Future<bool> onFetchCodeClick() async {
    final account = state.account;
    bool canSend = false;
    if (account.isNotEmpty) {
      if (i18nBook.isCurrentChinese) {
        canSend = account.isChineseMobileNumber;
      }
    }
    if (!canSend) {
      formEditState.accountEdited = true;
      PromptBox.toast("请输入正确的手机号");
      return false;
    }

    hideFormValidator();
    unfocusForm();

    // setBusy(i18nBook.auth.sendingVerificationCode.t);
    final isMobileAccount = account.isChineseMobileNumber;
    bool valid = false;
    try {
      if (isMobileAccount) {
        valid = await RPCBridge.ins.rpc.login.sendSMSVerificationCodeAsync(
          SendSMSVerificationCodeRequest(userPhone: account),
        );
      }
      if (valid) {
        PromptBox.toast(i18nBook.auth.toast4VerifyCodeSendSuccess.t);
        return true;
      }
    } catch (e) {
      logger.e(
          "User registration：Send${isMobileAccount ? 'short message' : 'email'} verification code exception",
          e);
    }
    PromptBox.toast(i18nBook.auth.toast4VerifyCodeSendFail.t);
    // busy = false;
    return false;
  }

  /// 选中的测量项
  FWidget _buildSlected(ItemMetaDTO e) {
    return FGestureDetector(
      name: "${e.name}",
      businessParent: widget,
      onTap: () {
        removeChooseMeasure(
          e.description ?? '',
        );
      },
      child: FStack(
        children: [
          FContainer(
            width: 180,
            height: 65,
            color: FTheme.ins.colorScheme.primary,
            child: FCenter(
              child: FColumn(
                mainAxisAlignment: MainAxisAlignment.center,
                children: [
                  if (!i18nBook.isCurrentEnglish)
                    FText(
                      measureLanguage.t('measure', e.description ?? ''),
                      style: const TextStyle(
                        color: Colors.white,
                        fontSize: 14,
                      ),
                    ),
                  FText(
                    e.description ?? '',
                    style: const TextStyle(
                      color: Colors.white,
                    ),
                  ),
                ],
              ),
            ),
            // ),
          ),

          /// 【TODO】 暂时屏蔽 buyStatus
          // if (e.status != null)
          //   FPositioned(
          //     bottom: 0,
          //     right: 0,
          //     child: _buildItemBuyStatusName(
          //       e.status ?? WorkingItemStatusEnum.Free,
          //     ),
          //   )
        ],
      ),
    );
  }

  // 构建测量项是否需要购买
  /// 【TODO】 暂时屏蔽 buyStatus
  // FWidget _buildItemBuyStatusName(WorkingItemStatusEnum buyStatus) {
  //   String name = "";
  //   Color textColor = Colors.white;

  //   switch (buyStatus) {
  //     case WorkingItemStatusEnum.Probation:
  //       name = i18nBook.measure.probation.t;
  //       textColor = Colors.yellow;
  //       break;
  //     case WorkingItemStatusEnum.Unpaid:
  //       name = i18nBook.measure.unpaid.t;
  //       textColor = Colors.red;
  //       break;
  //     // case WorkingItemStatusEnum.Purchased:
  //     //   name = i18nBook.measure.purchased.t;
  //     //   textColor = childButtonHighlight;
  //     //   break;
  //     case WorkingItemStatusEnum.Expired:
  //       name = i18nBook.measure.expired.t;
  //       textColor = Colors.red;
  //       break;
  //     default:
  //       break;
  //   }
  //   if (name.isNotEmpty) {
  //     return QuickFWidget(
  //       Tooltip(
  //         message: buyStatus == WorkingItemStatusEnum.Unpaid
  //             ? "如需继续使用该测量项，请联系Vinno申请试用或购买。"
  //             : "",
  //         padding: EdgeInsets.zero,
  //         margin: EdgeInsets.zero,
  //         child: Container(
  //           alignment: Alignment.center,
  //           padding: const EdgeInsets.symmetric(horizontal: 4, vertical: 2),
  //           color: textColor.withOpacity(0.4),
  //           // color: Colors.black.withOpacity(0.2),
  //           child: FText(
  //             name,
  //             style: const TextStyle(
  //               color: Colors.white,
  //               fontSize: 10,
  //             ),
  //           ),
  //         ),
  //       ),
  //     );
  //   }
  //   return const FSizedBox();
  // }

  /// 未选中的测量项
  FWidget _buildNoSlected(ItemMetaDTO e) {
    return FGestureDetector(
      name: "${e.name}",
      businessParent: widget,
      onTap: () {
        addChooseMeasure(
          e.description ?? '',
        );
      },
      child: FStack(
        children: [
          FContainer(
            width: 180,
            height: 65,
            decoration: BoxDecoration(
              border: Border.all(
                width: 0.5,
                color: FTheme.ins.colorScheme.primary,
              ),
            ),
            child: FCenter(
              child: FColumn(
                mainAxisAlignment: MainAxisAlignment.center,
                children: [
                  if (!i18nBook.isCurrentEnglish)
                    FText(
                      measureLanguage.t('measure', e.description ?? ''),
                      style: const TextStyle(
                        color: Colors.black,
                        fontSize: 14,
                      ),
                    ),
                  FText(
                    e.description ?? '',
                    style: const TextStyle(
                      color: Colors.black,
                    ),
                  ),
                ],
              ),
            ),
          ),

          /// 【TODO】 暂时屏蔽 buyStatus
          // if (e.status != null)
          //   FPositioned(
          //     bottom: 0,
          //     right: 0,
          //     child: _buildItemBuyStatusName(
          //       e.status ?? WorkingItemStatusEnum.Free,
          //     ),
          //   )
        ],
      ),
    );
  }

  FWidget _buildGroupBody(ItemMetaGroup e) {
    return FContainer(
      padding: const EdgeInsets.only(left: 15),
      margin: const EdgeInsets.symmetric(vertical: 10),
      child: FWrap(
          spacing: 16.0, // 主轴(水平)方向间距
          runSpacing: 8.0, // 纵轴（垂直）方向间距
          direction: Axis.horizontal,
          crossAlignment: WrapCrossAlignment.center,
          alignment: WrapAlignment.start,
          children: e.itemMeta.map((e) {
            if (widget.chooseMeasureList.contains(
              e.description,
            )) {
              return FMouseRegion(
                cursor: SystemMouseCursors.click,
                child: _buildSlected(e),
              );
            } else {
              return FMouseRegion(
                cursor: SystemMouseCursors.click,
                child: _buildNoSlected(e),
              );
            }
          }).toList()),
    );
  }
}
